#line 120 "../mdcode.mdc"
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "mdcode.h"

#line 860 "../mdcode.mdc"
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <string.h>

#line 832 "../mdcode.mdc"
static void copy_fname(char *name, int space, struct text t)
{
	char *sec = t.txt;
	int len = t.len;
	name[0] = 0;
	if (len < 5 || strncmp(sec, "File:", 5) != 0)
		return;
	sec += 5;
	len -= 5;
	while (len && sec[0] == ' ') {
		sec++;
		len--;
	}
	if (len >= space)
		len = space - 1;
	strncpy(name, sec, len);
	name[len] = 0;
}

#line 867 "../mdcode.mdc"
static int errs;
static void pr_err(char *msg)
{
	errs++;
	fprintf(stderr, "%s\n", msg);
}

int main(int argc, char *argv[])
{
	int fd;
	size_t len;
	char *file;
	struct section *table, *s, *prev;

	errs = 0;
	if (argc != 2) {
		fprintf(stderr, "Usage: mdcode file.mdc\n");
		exit(2);
	}
	fd = open(argv[1], O_RDONLY);
	if (fd < 0) {
		fprintf(stderr, "mdcode: cannot open %s: %s\n",
			argv[1], strerror(errno));
		exit(1);
	}
	len = lseek(fd, 0, 2);
	file = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
	table = code_extract(file, file+len, pr_err);

	for (s = table; s;
		(code_free(s->code), prev = s, s = s->next, free(prev))) {
		FILE *fl;
		char fname[1024];
		if (strncmp(s->section.txt, "Example:", 8) == 0)
			continue;
		if (strncmp(s->section.txt, "File:", 5) != 0) {
			fprintf(stderr, "Unreferenced section is not a file name: %.*s\n",
				s->section.len, s->section.txt);
			errs++;
			continue;
		}
		copy_fname(fname, sizeof(fname), s->section);
		if (fname[0] == 0) {
			fprintf(stderr, "Missing file name at:%.*s\n",
				s->section.len, s->section.txt);
			errs++;
			continue;
		}
		fl = fopen(fname, "w");
		if (!fl) {
			fprintf(stderr, "Cannot create %s: %s\n",
				fname, strerror(errno));
			errs++;
			continue;
		}
		code_node_print(fl, s->code, argv[1]);
		fclose(fl);
	}
	exit(!!errs);
}

#line 128 "../mdcode.mdc"

